/*
 * Created on Jun 15, 2004
 *
 */
package gov.grants.apply.struts.actions;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicantIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicationInformationType;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest_ApplicationFilter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListRequest_ApplicationFilter_Filter;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationListResponse;
import gov.grants.apply.soap.util.SoapUtils;
import gov.grants.apply.struts.Constants;
import gov.grants.apply.struts.forms.GetApplicationListForm;
import gov.grants.apply.struts.util.Utils;
import gov.grants.apply.valueObject.ApplicationListInformation;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * Action class for getting application list.
 * 
 * @author Feng Qian
 * @author Brian Husted
 * @author Mark Sommer
 *
 */

public class GetApplicationListAction extends Action implements Constants {

	static final Logger log = Logger.getLogger(GetApplicationListAction.class
			.getName());

	/**
	 * Executes an Action.
	 * @param mapping the ActionMapping.
	 * @param aform the ActionForm.
	 * @param request the HttpServletRequest.
	 * @param response the HttpServletResponse.
	 * @return the ActionForward.
	 * @throws Exception.
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		
		ApplicantIntegrationSoapBindingStub stub = SoapUtils.getApplicantSoapStub();
		

		_GetApplicationListResponse getApplicationListResponse = stub
				.getApplicationList(createRequest((GetApplicationListForm) form));
		log.debug("Adding application info");
		request.getSession().setAttribute(APPLICATION_INFO,
				formatReply(getApplicationListResponse));

		return mapping.findForward(ACTION_SUCCESS);

	}
	/**
	 * Creates an application list request
	 * 
	 * @param form the GetApplicationListForm
	 * @return the _GetApplicationListRequest
	 */
	protected _GetApplicationListRequest createRequest(
			GetApplicationListForm form) {

		List list = new ArrayList();
		GetApplicationListForm appListForm = (GetApplicationListForm) form;
		log.debug("Getting Filter Values");
		if (appListForm.getStatus() != null) {
			_GetApplicationListRequest_ApplicationFilter applicationFilter_status = new _GetApplicationListRequest_ApplicationFilter();
			applicationFilter_status
					.setFilter(_GetApplicationListRequest_ApplicationFilter_Filter.Status);
			applicationFilter_status.setFilterValue(appListForm.getStatus());
			log.debug("Adding a Status filter of: " + appListForm.getStatus());
			list.add(applicationFilter_status);
		}

		if (appListForm.getCFDANum() != null) {
			_GetApplicationListRequest_ApplicationFilter applicationFilter_CFDANum = new _GetApplicationListRequest_ApplicationFilter();
			applicationFilter_CFDANum
					.setFilter(_GetApplicationListRequest_ApplicationFilter_Filter.CFDANumber);
			applicationFilter_CFDANum.setFilterValue(appListForm.getCFDANum());
			log
					.debug("Adding a CFDANum filter of: "
							+ appListForm.getCFDANum());
			list.add(applicationFilter_CFDANum);
		}

		if (appListForm.getOpportunityID() != null) {
			_GetApplicationListRequest_ApplicationFilter applicationFilter_OpportunityID = new _GetApplicationListRequest_ApplicationFilter();
			applicationFilter_OpportunityID
					.setFilter(_GetApplicationListRequest_ApplicationFilter_Filter.OpportunityID);
			applicationFilter_OpportunityID.setFilterValue(appListForm
					.getOpportunityID());
			log.debug("Adding a OppId filter of: "
					+ appListForm.getOpportunityID());
			list.add(applicationFilter_OpportunityID);
		}
		
		if (appListForm.getSubmissionTitle() != null) {
			_GetApplicationListRequest_ApplicationFilter applicationFilter_SubmissionTitle = new _GetApplicationListRequest_ApplicationFilter();
			applicationFilter_SubmissionTitle
					.setFilter(_GetApplicationListRequest_ApplicationFilter_Filter.SubmissionTitle);
			applicationFilter_SubmissionTitle.setFilterValue(appListForm
					.getSubmissionTitle());
			log.debug("Adding a SubmissionTitle filter of: "
					+ appListForm.getOpportunityID());
			list.add(applicationFilter_SubmissionTitle);
		}
		
		

		_GetApplicationListRequest getApplicationListRequest = new _GetApplicationListRequest();
		getApplicationListRequest
				.setApplicationFilter((_GetApplicationListRequest_ApplicationFilter[]) list
						.toArray(new _GetApplicationListRequest_ApplicationFilter[0]));
		return getApplicationListRequest;
	}

	/**
	 * 
	 * Formats a reply
	 * 
	 * @param the getApplicationListResponse
	 * @return An List of the ApplicationInformationType
	 * @throws Exception
	 */
	protected List formatReply(
			_GetApplicationListResponse getApplicationListResponse)
			throws Exception {
		ArrayList applicationList = new ArrayList();

		ApplicationInformationType applicationInformationType = null;

		if (getApplicationListResponse != null) {
			ApplicationInformationType[] application = getApplicationListResponse
					.getApplicationInformation();

			if (application != null && application.length > 0) {
				log.debug("The server returned " + " " + application.length
						+ " applications");
				for (int i = 0; i < application.length; i++) {
					ApplicationListInformation wrapper = new ApplicationListInformation();
					if (application[i].getCFDANumber() != null) {
						wrapper.setCFDANumber(application[i].getCFDANumber()
								.getValue());
					}
					if (application[i].getOpportunityID() != null) {
						wrapper.setOpportunityID(application[i]
								.getOpportunityID().getValue());
					}
					if (application[i].getCompetitionID() != null) {
						wrapper.setCompetitionID(application[i]
								.getCompetitionID().getValue());
					}
					if (application[i].getGrants_govTrackingNumber() != null) {
						wrapper.setGrantsGovTrackingNumber(application[i]
								.getGrants_govTrackingNumber().getValue());
					}
					if (application[i].getGrantsGovApplicationStatus() != null) {
						wrapper.setStatus(application[i]
								.getGrantsGovApplicationStatus().getValue());
					}
					if (application[i].getStatusDateTime() != null) {
						wrapper.setStatusDate(Utils.formatDate(application[i]
								.getStatusDateTime().getTime()));
					}
					if (application[i].getReceivedDateTime() != null) {
						wrapper.setReceivedDateTime(Utils
								.formatDate(application[i]
										.getReceivedDateTime().getTime()));

					}
					if (application[i].getAgencyTrackingNumber() != null) {
						wrapper.setAgencyTrackingNumber(application[i]
								.getAgencyTrackingNumber().getValue());
					}
					if (application[i].getSubmissionTitle() != null) {
						wrapper.setSubmissionTitle(application[i]
								.getSubmissionTitle().getValue());
					}
					
					

					applicationList.add(wrapper);
				}
			}
		}
		return applicationList;
	}

}