/*
 * Created on November 8, 2004
 *
 */
package gov.grants.apply.struts.actions;

import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0.ApplicantIntegrationSoapBindingStub;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailRequest;
import gov.grants.apply.WebServices.ApplicantIntegrationServices_V1_0._GetApplicationStatusDetailResponse;
import gov.grants.apply.soap.util.SoapUtils;
import gov.grants.apply.struts.Constants;
import gov.grants.apply.struts.forms.GetApplicationStatusDetailForm;
import gov.grants.apply.system.Global_V1_0.StringMin1Max240Type;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionError;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

/**
 * Action class for getting detailed application status.
 * 
 * @author Mark Sommer
 *  
 */

public class GetApplicationStatusDetailAction extends Action
		implements
			Constants {

	static final Logger log = Logger
			.getLogger(GetApplicationStatusDetailAction.class.getName());

	/**
	 * Executes an Action.
	 * 
	 * @param mapping
	 *            the ActionMapping.
	 * @param aform
	 *            the ActionForm.
	 * @param request
	 *            the HttpServletRequest.
	 * @param response
	 *            the HttpServletResponse.
	 * @return the ActionForward.
	 * @throws Exception.
	 */
	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {

		GetApplicationStatusDetailForm appStatusDetailForm = (GetApplicationStatusDetailForm) form;

		//	=============================================================
		//	 Perform Form Validation
		//	=============================================================
		if (validateRequest(appStatusDetailForm) != true) {
			ActionErrors errors = new ActionErrors();
			errors.add(ActionErrors.GLOBAL_ERROR, new ActionError(
					"errors.trackingid.required"));
			saveErrors(request, errors);
			log.warn("No Grants.gov Tracking Number was entered.");

			return mapping.findForward(ACTION_CONTINUE);
		}

		//	=============================================================
		//	 Declaration of Variables
		//	=============================================================
		ApplicantIntegrationSoapBindingStub stub = SoapUtils
				.getApplicantSoapStub();

		_GetApplicationStatusDetailResponse getApplicationStatusDetailResponse = stub
				.getApplicationStatusDetail(createRequest((GetApplicationStatusDetailForm) form));

		//	===============================================================
		//	 Check DetailedStatus returned from _GetApplicationStatusDetailProcess
		// class converted to an Object
		//	================================================================

		Object statusDetail = getApplicationStatusDetailResponse
				.getDetailedStatus();

		//=================================================================
		// When a Record is found or not found
		//=================================================================

		log.debug("Adding Grants.gov Tracking ID and Status Details");
		request.getSession().setAttribute(
				GG_TRACKING_ID,
				getApplicationStatusDetailResponse
						.getGrants_govTrackingNumber().getValue());
		request.getSession().setAttribute(STATUS_DETAIL,
				statusDetail.toString());
		return mapping.findForward(ACTION_SUCCESS);

	}

	/**
	 * Creates an application status detail request
	 * 
	 * @param form
	 *            the GetApplicationStatusDetailForm
	 * @return the _GetApplicationStatusDetailRequest
	 */
	protected _GetApplicationStatusDetailRequest createRequest(
			GetApplicationStatusDetailForm form) {

		log.debug("Setting Grants.gov Tracking Number");

		_GetApplicationStatusDetailRequest request = new _GetApplicationStatusDetailRequest();
		request.setGrants_govTrackingNumber(new StringMin1Max240Type(form
				.getTrackingID().trim()));
		return request;
	}

	/**
	 * Validates data
	 * 
	 * @param mapping
	 *            the ActionMapping
	 * @param request
	 *            the HttpServletRequest
	 * @param x
	 *            String
	 * @return a boolean
	 */
	private static boolean validateRequest(GetApplicationStatusDetailForm form) {
		String trackingId = null;
		trackingId = form.getTrackingID().trim();
		if ((trackingId == null) || (trackingId.length() <= 0)) {
			return false;
		} else {
			log.debug("GG Tracking ID Found: " + trackingId.trim() + ". ");
			return true;
		}

	}

}